import { AD_GET_BASIC_ROUTE_CONTROLLER } from '@modules/ad/ad.di-tokens';
import { CarpoolRoute } from '@modules/ad/core/application/types/carpool-route.type';
import { Point } from '@modules/ad/core/application/types/point.type';
import { Role } from '@modules/ad/core/domain/ad.types';
import { CarpoolRouteProvider } from '@modules/ad/infrastructure/carpool-route-provider';
import { GetBasicRouteControllerPort } from '@modules/geography/core/application/ports/get-basic-route-controller.port';
import { Test, TestingModule } from '@nestjs/testing';

const originPoint: Point = {
  lat: 48.689445,
  lon: 6.17651,
};
const destinationPoint: Point = {
  lat: 48.8566,
  lon: 2.3522,
};
const additionalPoint: Point = {
  lon: 48.7566,
  lat: 4.4498,
};

const mockGetBasicRouteController: GetBasicRouteControllerPort = {
  get: jest
    .fn()
    .mockImplementationOnce(() => ({
      distance: 350101,
      duration: 14422,
      fwdAzimuth: 273,
      backAzimuth: 93,
      distanceAzimuth: 336544,
      points: [
        {
          lon: 6.1765102,
          lat: 48.689445,
        },
        {
          lon: 4.984578,
          lat: 48.725687,
        },
        {
          lon: 2.3522,
          lat: 48.8566,
        },
      ],
    }))
    .mockImplementationOnce(() => ({
      distance: 350102,
      duration: 14423,
      fwdAzimuth: 273,
      backAzimuth: 93,
      distanceAzimuth: 336545,
      points: [
        {
          lon: 6.1765103,
          lat: 48.689446,
        },
        {
          lon: 4.984579,
          lat: 48.725688,
        },
        {
          lon: 2.3523,
          lat: 48.8567,
        },
      ],
    }))
    .mockImplementationOnce(() => ({
      distance: 350100,
      duration: 14421,
      fwdAzimuth: 273,
      backAzimuth: 93,
      distanceAzimuth: 336543,
      points: [
        {
          lon: 6.1765101,
          lat: 48.689444,
        },
        {
          lon: 4.984577,
          lat: 48.725686,
        },
        {
          lon: 2.3521,
          lat: 48.8565,
        },
      ],
    }))
    .mockImplementationOnce(() => ({
      distance: 350107,
      duration: 14427,
      fwdAzimuth: 273,
      backAzimuth: 93,
      distanceAzimuth: 336548,
      points: [
        {
          lon: 6.1765101,
          lat: 48.689444,
        },
        {
          lon: 4.984577,
          lat: 48.725686,
        },
        {
          lon: 2.3521,
          lat: 48.8565,
        },
      ],
    }))
    .mockImplementationOnce(() => ({
      distance: 350108,
      duration: 14428,
      fwdAzimuth: 273,
      backAzimuth: 93,
      distanceAzimuth: 336548,
      points: [
        {
          lon: 6.1765101,
          lat: 48.689444,
        },
        {
          lon: 4.984577,
          lat: 48.725686,
        },
        {
          lon: 2.3521,
          lat: 48.8565,
        },
      ],
    }))
    .mockImplementationOnce(() => []),
};

describe('Carpool route provider', () => {
  let carpoolRouteProvider: CarpoolRouteProvider;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        CarpoolRouteProvider,
        {
          provide: AD_GET_BASIC_ROUTE_CONTROLLER,
          useValue: mockGetBasicRouteController,
        },
      ],
    }).compile();

    carpoolRouteProvider =
      module.get<CarpoolRouteProvider>(CarpoolRouteProvider);
  });

  it('should be defined', () => {
    expect(carpoolRouteProvider).toBeDefined();
  });

  it('should provide a carpool route for a driver only', async () => {
    const carpoolRoute: CarpoolRoute = await carpoolRouteProvider.getBasic(
      [Role.DRIVER],
      [
        {
          position: 0,
          ...originPoint,
        },
        {
          position: 1,
          ...destinationPoint,
        },
      ],
    );
    expect(carpoolRoute.driverDistance).toBe(350101);
    expect(carpoolRoute.passengerDuration).toBeUndefined();
  });

  it('should provide a carpool route for a passenger only', async () => {
    const carpoolRoute: CarpoolRoute = await carpoolRouteProvider.getBasic(
      [Role.PASSENGER],
      [
        {
          position: 0,
          ...originPoint,
        },
        {
          position: 1,
          ...destinationPoint,
        },
      ],
    );
    expect(carpoolRoute.passengerDistance).toBe(350102);
    expect(carpoolRoute.driverDuration).toBeUndefined();
  });

  it('should provide a simple carpool route for a driver and passenger', async () => {
    const carpoolRoute: CarpoolRoute = await carpoolRouteProvider.getBasic(
      [Role.DRIVER, Role.PASSENGER],
      [
        {
          position: 0,
          ...originPoint,
        },
        {
          position: 1,
          ...destinationPoint,
        },
      ],
    );
    expect(carpoolRoute.driverDuration).toBe(14421);
    expect(carpoolRoute.passengerDistance).toBe(350100);
  });

  it('should provide a complex carpool route for a driver and passenger', async () => {
    const carpoolRoute: CarpoolRoute = await carpoolRouteProvider.getBasic(
      [Role.DRIVER, Role.PASSENGER],
      [
        {
          position: 0,
          ...originPoint,
        },
        {
          position: 1,
          ...additionalPoint,
        },
        {
          position: 2,
          ...destinationPoint,
        },
      ],
    );
    expect(carpoolRoute.driverDistance).toBe(350107);
    expect(carpoolRoute.passengerDuration).toBe(14428);
  });
});
